@@ -6,7 +6,6 @@
 
 import os
 import pathlib
-import math
 
 from direct.showbase.DirectObject import DirectObject
 from direct.gui import DirectGuiGlobals as DGG
@@ -23,13 +22,20 @@ from panda3d.core import (
     TextNode
 )
 
-
 from panda3d.core import PGButton, MouseButton
 DGG.MWUP = PGButton.getPressPrefix() + MouseButton.wheel_up().getName() + '-'
 DGG.MWDOWN = PGButton.getPressPrefix() + MouseButton.wheel_down().getName() + '-'
 
+from .SymbolView import generate as symbolViewGenerate
+from .DetailView import generate as detailViewGenerate
+
+VIEWTYPE = {
+    "Symbol":symbolViewGenerate,
+    "Detail":detailViewGenerate
+}
+
 class DirectFolderBrowser(DirectObject):
-    def __init__(self, command, fileBrowser=False, defaultPath="~", defaultFilename="unnamed.txt", fileExtensions=[], tooltip=None, iconDir=None):
+    def __init__(self, command, fileBrowser=False, defaultPath="~", defaultFilename="unnamed.txt", fileExtensions=[], tooltip=None, iconDir=None, parent=None):
         """
         A simple file and folder browser
 
@@ -39,40 +45,59 @@ class DirectFolderBrowser(DirectObject):
         defaultFilename: The filename that will be set by default, only usefull if fileBrowser is True
         fileExtensions: A list of extensions. Only files with those extensions will be shown. Only usefull if fileBrowser is True
         tooltip: An instance of the Tooltip class to display tooltips for certain parts of the editor
-        iconDir: A directory path that contains replacement images. It must contain all required images.
+        iconDir: A directory path that contains replacement images. It must contain all required images which are:
+            File.png
+            Folder.png
+            FolderNew.png
+            FolderShowHidden.png
+            FolderUp.png
+            Reload.png
+        parent: Another DirectGUI element which has pixel2d as root parent.
+            The browser frame is placed centered so a frame for example should have equal sizes in horizontal and vertical directions
+            e.g. frameSize=(-250,250,-200,200)
         """
         self.tt = tooltip
         self.command = command
         self.showFiles = fileBrowser
         self.fileExtensions = fileExtensions
         self.showHidden = False
+        self.parent = parent
         if iconDir is None:
             self.iconDir = str(pathlib.PurePosixPath(__file__).parent) + "/icons"
         else:
             self.iconDir = iconDir
+        self.selectedViewType = "Symbol"
 
         self.currentPath = os.path.expanduser(defaultPath)
         if not os.path.exists(self.currentPath):
             self.currentPath = os.path.expanduser("~")
         self.previousPath = self.currentPath
 
-        self.screenWidthPx = base.getSize()[0]
+        if self.parent is None:
+            self.parent = base.pixel2d
+            self.screenWidthPx = base.getSize()[0]
+            self.screenHeightPx = base.getSize()[1]
+            self.position = LPoint3f(base.getSize()[0]/2, 0, -base.getSize()[1]/2)
+        else:
+            self.screenWidthPx = self.parent.getWidth()
+            self.screenHeightPx = self.parent.getHeight()
+            self.position = LPoint3f(0)
         self.screenWidthPxHalf = self.screenWidthPx * 0.5
-        self.screenHeightPx = base.getSize()[1]
         self.screenHeightPxHalf = self.screenHeightPx * 0.5
 
         self.mainFrame = DirectFrame(
             relief=1,
             frameSize=(-self.screenWidthPxHalf,self.screenWidthPxHalf,-self.screenHeightPxHalf,self.screenHeightPxHalf),
             frameColor=(1, 1, 1, 1),
-            pos=LPoint3f(base.getSize()[0]/2, 0, -base.getSize()[1]/2),
-            parent=base.pixel2d,
+            pos=self.position,
+            parent=self.parent,
             state=DGG.NORMAL,
         )
 
-        self.pathRightMargin = 153
-        self.pathEntryWidth = self.screenWidthPx - self.pathRightMargin
+        self.pathRightMargin = 155 # NOTE: Add 28 for each button to the right + 15px margin
+        self.pathEntryWidth = self.screenWidthPx - self.pathRightMargin - 28
 
+        # The path entry on top of the window
         self.pathEntry = DirectEntry(
             parent=self.mainFrame,
             relief=DGG.SUNKEN,
@@ -89,7 +114,13 @@ class DirectFolderBrowser(DirectObject):
             focusOutCommand=base.messenger.send,
             focusOutExtraArgs=["reregisterKeyboardEvents"],
         )
-        x = self.pathEntryWidth/2-28
+
+        # ----------------
+        # CONTROL BUTTONS
+        # ----------------
+        x = self.screenWidthPxHalf - self.pathRightMargin + 18
+
+        # RELOAD
         self.btnReload = DirectButton(
             parent=self.mainFrame,
             relief=1,
@@ -109,6 +140,8 @@ class DirectFolderBrowser(DirectObject):
         if self.tt is not None:
             self.btnReload.bind(DGG.ENTER, self.tt.show, ["Reload Folder"])
             self.btnReload.bind(DGG.EXIT, self.tt.hide)
+
+        # MOVE UP ONE FOLDER
         x += 28
         self.btnFolderUp = DirectButton(
             parent=self.mainFrame,
@@ -129,6 +162,8 @@ class DirectFolderBrowser(DirectObject):
         if self.tt is not None:
             self.btnFolderUp.bind(DGG.ENTER, self.tt.show, ["Move up one level"])
             self.btnFolderUp.bind(DGG.EXIT, self.tt.hide)
+
+        # CREATE NEW FOLDER
         x += 28
         self.btnFolderNew = DirectButton(
             parent=self.mainFrame,
@@ -149,6 +184,8 @@ class DirectFolderBrowser(DirectObject):
         if self.tt is not None:
             self.btnFolderNew.bind(DGG.ENTER, self.tt.show, ["Create new folder"])
             self.btnFolderNew.bind(DGG.EXIT, self.tt.hide)
+
+        # SHOW HIDDEN FOLDERS
         x += 28
         self.btnFolderShowHidden = DirectButton(
             parent=self.mainFrame,
@@ -170,6 +207,31 @@ class DirectFolderBrowser(DirectObject):
             self.btnFolderShowHidden.bind(DGG.ENTER, self.tt.show, ["Show/Hide hidden files and folders"])
             self.btnFolderShowHidden.bind(DGG.EXIT, self.tt.hide)
 
+        # TOGGLE VIEW TYPE
+        x += 28
+        self.btnViewType = DirectButton(
+            parent=self.mainFrame,
+            relief=1,
+            frameColor = (
+                (0.8, 0.8, 0.8, 1), # Normal
+                (0.9, 0.9, 1, 1), # Click
+                (0.8, 0.8, 1, 1), # Hover
+                (0.5, 0.5, 0.5, 1)), # Disabled
+            frameSize=(-14, 14, -10, 18),
+            pos=LPoint3f(x, 0, self.screenHeightPxHalf - 25),
+            command=self.toggleViewType,
+            image=f"{self.iconDir}/ViewTypeSymbol.png",
+            image_scale=14,
+            image_pos=(0,0,4),
+        )
+        self.btnViewType.setTransparency(TransparencyAttrib.M_multisample)
+        if self.tt is not None:
+            self.btnViewType.bind(DGG.ENTER, self.tt.show, ["Toggle view between Symbols and Detail list"])
+            self.btnViewType.bind(DGG.EXIT, self.tt.hide)
+
+        # --------------
+        # CONTENT FRAME
+        # --------------
         color = (
             (0.8, 0.8, 0.8, 1), # Normal
             (0.9, 0.9, 1, 1), # Click
@@ -202,6 +264,7 @@ class DirectFolderBrowser(DirectObject):
         self.container.bind(DGG.MWDOWN, self.scroll, [0.01])
         self.container.bind(DGG.MWUP, self.scroll, [-0.01])
 
+        # ACCEPT BUTTON
         self.btnOk = DirectButton(
             parent=self.mainFrame,
             relief=1,
@@ -217,6 +280,8 @@ class DirectFolderBrowser(DirectObject):
             command=command,
             extraArgs=[1],
         )
+
+        # CANCEL BUTTON
         self.btnCancel = DirectButton(
             parent=self.mainFrame,
             relief=1,
@@ -233,6 +298,7 @@ class DirectFolderBrowser(DirectObject):
             extraArgs=[0]
         )
 
+        # SELECTED FILE ENTRY FIELD
         if self.showFiles:
             self.txtFileName = DirectEntry(
                 parent=self.mainFrame,
@@ -251,6 +317,10 @@ class DirectFolderBrowser(DirectObject):
                 focusOutExtraArgs=["reregisterKeyboardEvents"],
             )
 
+        # ------------------
+        # CREATE NEW FOLDER
+        # ------------------
+        # FRAME FOR CREATING NEW FOLDER
         self.newFolderFrame = DirectFrame(
             parent=self.mainFrame,
             relief=1,
@@ -258,6 +328,8 @@ class DirectFolderBrowser(DirectObject):
             pos=LPoint3f(0, 0, self.screenHeightPxHalf-55),
             frameColor=(0.5,0.5,0.5,1),
         )
+
+        # LABEL FOR NEW FOLDER NAME ENTRY
         self.txtNewFolderName = DirectLabel(
             parent=self.newFolderFrame,
             text="New Folder Name",
@@ -266,6 +338,8 @@ class DirectFolderBrowser(DirectObject):
             text_align=TextNode.ALeft,
             pos=(-self.screenWidthPxHalf+15, 0, -3),
         )
+
+        # ENTRY FOR THE NEW FOLDER NAME
         self.folderName = DirectEntry(
             parent=self.newFolderFrame,
             relief=DGG.SUNKEN,
@@ -282,6 +356,8 @@ class DirectFolderBrowser(DirectObject):
             focusOutCommand=base.messenger.send,
             focusOutExtraArgs=["reregisterKeyboardEvents"],
         )
+
+        # ACCEPT BUTTON FOR THE CREATE NEW FOLDER
         self.btnCreate = DirectButton(
             parent=self.newFolderFrame,
             relief=1,
@@ -297,17 +373,24 @@ class DirectFolderBrowser(DirectObject):
             command=self.folderCreate,
             extraArgs=[0]
         )
+        # Hide the create new folder frame by default
         self.newFolderFrame.hide()
 
+        # ---------------
+        # UPDATE CONTENT
+        # ---------------
+        # Initial loading of the files and folders of the current path
         self.folderReload()
 
         # handle window resizing
         self.prevScreenSize = base.getSize()
-        self.accept("window-event", self.windowEventHandler)
+        if self.parent is base.pixel2d:
+            self.accept("window-event", self.windowEventHandler)
 
     def show(self):
         self.mainFrame.show()
-        self.accept("window-event", self.windowEventHandler)
+        if self.parent is None:
+            self.accept("window-event", self.windowEventHandler)
 
     def hide(self):
         self.ignore("window-event")
@@ -352,55 +435,7 @@ class DirectFolderBrowser(DirectObject):
             return
 
         # start position for the folders and files
-        xPos = -self.screenWidthPxHalf + 20 + 50 - 110
-        zPos = self.screenHeightPxHalf-60-40
-
-        dirList = []
-        fileList = []
-        unkList = []
-
-        for entry in content:
-            if entry.name.startswith(".") and not self.showHidden:
-                continue
-            if entry.is_dir():
-                dirList.append(entry)
-            elif entry.is_file() and self.showFiles:
-                if len(self.fileExtensions) > 0:
-                    if os.path.splitext(entry.name)[1] in self.fileExtensions:
-                        fileList.append(entry)
-                else:
-                    fileList.append(entry)
-            elif self.showFiles:
-                unkList.append(entry)
-
-        def moveNext(entry):
-            nonlocal xPos
-            nonlocal zPos
-            if entry.is_dir() or self.showFiles:
-                if xPos + 110 > self.screenWidthPxHalf - 45:
-                    # move to the next line if we hit the right border (incl. scrollbar size)
-                    xPos = -self.screenWidthPxHalf + 20 + 50
-                    zPos -= 110
-                else:
-                    # move right the next position
-                    xPos += 110
-
-        def getKey(item):
-            return item.name.lower()
-
-        for entry in sorted(dirList, key=getKey):
-            moveNext(entry)
-            self.__createFolder(entry, xPos, zPos)
-        for entry in sorted(fileList, key=getKey):
-            moveNext(entry)
-            self.__createFile(entry.name, xPos, zPos)
-        for entry in sorted(unkList, key=getKey):
-            moveNext(entry)
-            self.__createUnknown(entry.name, xPos, zPos)
-
-        # recalculate the canvas size
-        self.container["canvasSize"] = (-self.screenWidthPxHalf+31, self.screenWidthPxHalf-15, zPos-90, self.screenHeightPxHalf-50)
-        self.container.setCanvasSize()
+        VIEWTYPE[self.selectedViewType](self, content)
 
     def folderUp(self):
         self.previousPath = self.currentPath
@@ -427,6 +462,16 @@ class DirectFolderBrowser(DirectObject):
         self.showHidden = not self.showHidden
         self.folderReload()
 
+    def toggleViewType(self):
+        if self.selectedViewType == "Symbol":
+            self.selectedViewType = "Detail"
+            self.btnViewType["image"] = f"{self.iconDir}/ViewTypeDetail.png"
+        else:
+            self.selectedViewType = "Symbol"
+            self.btnViewType["image"] = f"{self.iconDir}/ViewTypeSymbol.png"
+
+        self.folderReload()
+
     def folderCreate(self, path=""):
         try:
             os.makedirs(os.path.join(self.currentPath, self.folderName.get(True)))
@@ -435,93 +480,6 @@ class DirectFolderBrowser(DirectObject):
         self.newFolderFrame.hide()
         self.folderReload()
 
-    def __createFolder(self, entry, xPos, zPos):
-        name = entry.name
-        if len(entry.name) > 10:
-            name = ""
-            for i in range(max(math.ceil(len(entry.name)/10), 4)):
-                name += entry.name[i*10:i*10+10]+"\n"
-            name = name[:-1]
-            if math.ceil(len(entry.name)/10) > 4:
-                name += "..."
-        btn = DirectButton(
-            parent=self.container.getCanvas(),
-            image=f"{self.iconDir}/Folder.png",
-            image_scale=35,
-            relief=1,
-            frameColor = (
-                (0.9, 0.9, 0.9, 0), # Normal
-                (0.95, 0.95, 1, 1), # Click
-                (0.9, 0.9, 1, 1), # Hover
-                (0.5, 0.5, 0.5, 1)), # Disabled
-            frameSize=(-40, 40, -40, 40),
-            pos=LPoint3f(xPos, 0, zPos),
-            text = name,
-            text_scale=12,
-            text_pos=(0,-40),
-            command=self.folderMoveIn,
-            extraArgs=[entry.path]
-        )
-        btn.bind(DGG.MWDOWN, self.scroll, [0.01])
-        btn.bind(DGG.MWUP, self.scroll, [-0.01])
-        btn.setTransparency(TransparencyAttrib.M_multisample)
-
-    def __createFile(self, filename, xPos, zPos):
-        name = filename
-        if len(filename) > 10:
-            name = ""
-            for i in range(min(math.ceil(len(filename)/10), 4)):
-                name += filename[i*10:i*10+10]+"\n"
-            name = name[:-1]
-            if math.ceil(len(filename)/10) > 4:
-                name += "..."
-        btn = DirectButton(
-            parent=self.container.getCanvas(),
-            image=f"{self.iconDir}/File.png",
-            image_scale=35,
-            relief=1,
-            frameColor = (
-                (0.9, 0.9, 0.9, 0), # Normal
-                (0.95, 0.95, 1, 1), # Click
-                (0.9, 0.9, 1, 1), # Hover
-                (0.5, 0.5, 0.5, 1)), # Disabled
-            frameSize=(-40, 40, -40, 40),
-            pos=LPoint3f(xPos, 0, zPos),
-            text = name,
-            text_scale=12,
-            text_pos=(0,-40),
-            command=self.txtFileName.set,
-            extraArgs=[filename]
-        )
-        btn.bind(DGG.MWDOWN, self.scroll, [0.01])
-        btn.bind(DGG.MWUP, self.scroll, [-0.01])
-        btn.setTransparency(TransparencyAttrib.M_multisample)
-
-    def __createUnknown(self, filename, xPos, zPos):
-        name = filename
-        if len(filename) > 10:
-            name = ""
-            for i in range(math.ceil(len(filename)/10)):
-                name += filename[i*10:i*10+10]+"\n"
-            name = name[:-1]
-        lbl = DirectLabel(
-            parent=self.container.getCanvas(),
-            image=f"{self.iconDir}/File.png",
-            image_scale=35,
-            image_color=(0.9,0.5,0.5,1),
-            relief=1,
-            frameColor = (0.7, 0.7, 0.7, 0),
-            frameSize=(-40, 40, -40, 40),
-            pos=LPoint3f(xPos, 0, zPos),
-            text = name,
-            text_scale=12,
-            text_pos=(0,-40),
-        )
-        lbl.bind(DGG.MWDOWN, self.scroll, [0.01])
-        lbl.bind(DGG.MWUP, self.scroll, [-0.01])
-        lbl.setTransparency(TransparencyAttrib.M_multisample)
-
-
     def windowEventHandler(self, window=None):
         if window != base.win:
             # This event isn't about our window.
@@ -541,13 +499,13 @@ class DirectFolderBrowser(DirectObject):
             self.mainFrame.setPos(self.screenWidthPx/2, 0, -self.screenHeightPx/2)
             self.mainFrame["frameSize"] = (-self.screenWidthPxHalf,self.screenWidthPxHalf,-self.screenHeightPxHalf,self.screenHeightPxHalf)
 
-            self.pathEntryWidth = self.screenWidthPx - self.pathRightMargin
+            self.pathEntryWidth = self.screenWidthPx - self.pathRightMargin - 28
             self.pathEntry.setPos(LPoint3f(-self.screenWidthPxHalf + 15, 0, self.screenHeightPxHalf - 25))
             self.pathEntry["width"] = self.pathEntryWidth/12
             self.pathEntry.resetFrameSize()
 
             # reposition top right icons
-            x = self.pathEntryWidth/2-28
+            x = self.screenWidthPxHalf - self.pathRightMargin + 14
             self.btnReload.setPos(LPoint3f(x, 0, self.screenHeightPxHalf - 25))
             x += 28
             self.btnFolderUp.setPos(pos=LPoint3f(x, 0, self.screenHeightPxHalf - 25))
@@ -555,6 +513,8 @@ class DirectFolderBrowser(DirectObject):
             self.btnFolderNew.setPos(pos=LPoint3f(x, 0, self.screenHeightPxHalf - 25))
             x += 28
             self.btnFolderShowHidden.setPos(pos=LPoint3f(x, 0, self.screenHeightPxHalf - 25))
+            x += 28
+            self.btnViewType.setPos(pos=LPoint3f(x, 0, self.screenHeightPxHalf - 25))
 
             # resize the browsing area
             self.container["frameSize"] = (-self.screenWidthPxHalf+10, self.screenWidthPxHalf-10, -self.screenHeightPxHalf+50, self.screenHeightPxHalf-50)
